package it.openutils.configuration.services;

import it.openutils.configuration.dataobjects.ConfigurationCode;
import it.openutils.configuration.dataobjects.ConfigurationDomain;

import java.util.List;

/**
 * @author fgiust
 * @version $Id: ConfigurationManager.java 179 2007-02-01 16:35:22Z fcarone $
 */
public interface ConfigurationManager
{

    /**
     * Returns the list of all esisting ConfigurationDomain
     * @return a list of ConfigurationDomain.
     */
    List<ConfigurationDomain> getConfigurationDomains();

    /**
     * Returns the list of ConfigurationCode associated to a given domain
     * @param domain domain
     * @return configuration configuration.
     */
    List<ConfigurationCode> getConfigurationCodes(String domain);

    /**
     * Returns the list of ConfigurationCode associated to a given domain
     * @param domain domain
     * @return configuration configuration.
     */
    List<String> getConfigurationCodesAsString(String domain);

    /**
     * Save a ConfigurationDomain
     * @param domain domain
     * @return domain ConfigurationDomain's instance.
     */
    ConfigurationDomain saveOrUpdateConfigurationDomain(ConfigurationDomain domain);

    /**
     * Save a ConfigurationCode
     * @param configurationCode configurationCode
     * @return configurationCode ConfigurationCode instance.
     */
    ConfigurationCode saveOrUpdateConfigurationCode(ConfigurationCode configurationCode);

    /**
     * Remove a ConfigurationDomain
     * @param domain domain
     */
    void removeConfigurationDomain(String domain);

    /**
     * Remove a ConfigurationCode
     * @param configurationCode configurationCode
     */
    void removeConfigurationCode(ConfigurationCode configurationCode);

    /**
     * @param domainId The configuration domani you want configuration codes for.
     * @return configuration domain by id
     */
    ConfigurationDomain getConfigurationDomain(String domainId);

    /**
     * @param domainId The configuration domani you want configuration codes for.
     * @param codeId The configuration code id.
     * @return The ConfigurationCode identified by the configuration code id.
     */
    ConfigurationCode getConfigurationCode(String domainId, String codeId);

}
