package it.openutils.configuration.services;

import it.openutils.configuration.dao.ConfigurationCodeDAO;
import it.openutils.configuration.dao.ConfigurationDomainDAO;
import it.openutils.configuration.dataobjects.ConfigurationCode;
import it.openutils.configuration.dataobjects.ConfigurationDomain;
import it.openutils.configuration.exceptions.ConfigurationDomainDeleteException;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;


/**
 * @author fgiust
 * @version $Id: ConfigurationManagerImpl.java 179 2007-02-01 16:35:22Z fcarone $
 */
public class ConfigurationManagerImpl implements ConfigurationManager
{

    /**
     * Data access object for ConfigurationCode.
     */
    private ConfigurationCodeDAO configurationCodeDAO;

    /**
     * Data access object for ConfigurationDomain.
     */
    private ConfigurationDomainDAO configurationDomainDAO;

    /**
     * Sets the configurationCodeDAO.
     * @param configurationCodeDAO the configurationCodeDAO to set
     */
    public void setConfigurationCodeDAO(ConfigurationCodeDAO configurationCodeDAO)
    {
        this.configurationCodeDAO = configurationCodeDAO;
    }

    /**
     * Sets the configurationDomainDAO.
     * @param configurationDomainDAO the configurationDomainDAO to set
     */
    public void setConfigurationDomainDAO(ConfigurationDomainDAO configurationDomainDAO)
    {
        this.configurationDomainDAO = configurationDomainDAO;
    }

    /**
     * {@inheritDoc}
     */
    public List<ConfigurationDomain> getConfigurationDomains()
    {

        return configurationDomainDAO.findAll();
    }

    /**
     * {@inheritDoc}
     */
    public List<ConfigurationCode> getConfigurationCodes(String domain)
    {
        ConfigurationDomain configurationDomain = configurationDomainDAO.loadIfAvailable(domain);
        if (configurationDomain == null)
        {
            return new ArrayList<ConfigurationCode>();
        }

        ConfigurationCode filter = new ConfigurationCode();
        filter.setConfigurationdomain(configurationDomain);

        return configurationCodeDAO.findFiltered(filter);
    }

    /**
     * {@inheritDoc}
     */
    public List<String> getConfigurationCodesAsString(String domain)
    {
        ConfigurationDomain configurationDomain = configurationDomainDAO.loadIfAvailable(domain);
        if (configurationDomain == null)
        {
            return new ArrayList<String>();
        }

        List<String> configurationCodes = new ArrayList<String>();
        List<ConfigurationCode> configurationCodesList = new ArrayList<ConfigurationCode>();

        ConfigurationCode filter = new ConfigurationCode();
        filter.setConfigurationdomain(configurationDomain);
        configurationCodesList = configurationCodeDAO.findFiltered(filter);

        for (ConfigurationCode code : configurationCodesList)
        {
            configurationCodes.add(code.getCode());
        }

        return configurationCodes;
    }

    /**
     * {@inheritDoc}
     */
    public ConfigurationDomain saveOrUpdateConfigurationDomain(ConfigurationDomain domain)
    {
        configurationDomainDAO.saveOrUpdate(domain);

        return domain;
    }

    /**
     * {@inheritDoc}
     */
    public ConfigurationCode saveOrUpdateConfigurationCode(ConfigurationCode configurationCode)
    {
        configurationDomainDAO.saveOrUpdate(configurationCode.getConfigurationdomain());
        configurationCodeDAO.saveOrUpdate(configurationCode);

        return configurationCode;
    }

    /**
     * {@inheritDoc}
     */
    public void removeConfigurationCode(ConfigurationCode configurationCode)
    {
        configurationCodeDAO.delete(configurationCode.getId());
    }

    /**
     * {@inheritDoc}
     */
    public void removeConfigurationDomain(String domain)
    {
        List<ConfigurationCode> configurationCodes = getConfigurationCodes(domain);
        if (configurationCodes.isEmpty())
        {
            configurationDomainDAO.delete(domain);
        }
        else
        {
            throw new ConfigurationDomainDeleteException("Cannot delete ConfigurationDomain with ConfigurationCodes.");
        }
    }

    /**
     * {@inheritDoc}
     */
    public ConfigurationDomain getConfigurationDomain(String domainId)
    {
        ConfigurationDomain domain = configurationDomainDAO.load(domainId);
        return domain;
    }

    /**
     * {@inheritDoc}
     */
    public ConfigurationCode getConfigurationCode(String domainId, String codeId)
    {
        List<ConfigurationCode> configurationCodes = getConfigurationCodes(domainId);
        for (ConfigurationCode configurationCode : configurationCodes)
        {
            if (StringUtils.equals(configurationCode.getCode(), codeId))
            {
                return configurationCode;
            }
        }
        return null;
    }

}
