package it.openutils.configuration.services;

import it.openutils.configuration.dataobjects.ReferenceCode;
import it.openutils.configuration.dataobjects.ReferenceDomain;

import java.util.List;
import java.util.Map;

/**
 * Manages reference code and domains.
 * @author fgiust
 * @version $Id: ReferenceManager.java 179 2007-02-01 16:35:22Z fcarone $
 */
public interface ReferenceManager
{

    /**
     * Returns the list of all esisting ReferenceDomain
     * @return a list of ReferenceDomain.
     */
    List<ReferenceDomain> getReferenceDomains();

    /**
     * Returns the list of Active reference codes associated to a given domain
     * @param domainId ReferenceDomain code
     * @return reference code values as an array of strings.
     */
    List<String> getReferenceCodesAsString(String domainId);

    /**
     * Return the domain with the given id
     * @param domainId ReferenceDomain code
     * @return ReferenceDomain
     */
    ReferenceDomain getReferenceDomain(String domainId);

    /**
     * Returns the list of all reference codes associated to a given domain referenceCodes
     * @param domainId ReferenceDomain code
     * @return reference codes.
     */
    List<ReferenceCode> getReferenceCodes(String domainId);

    /**
     * Returns the list of Active reference codes associated to a given domain
     * @param domainId ReferenceDomain code
     * @return reference codes.
     */
    Map<String, String> getReferenceCodesAsMap(String domainId);

    /**
     * Saves a given reference code.
     * @param code ReferenceCode instance
     * @return saved ReferenceCode instance.
     */
    ReferenceCode saveOrUpdateReferenceCode(ReferenceCode code);

    /**
     * Saves a given reference domain.
     * @param domain ReferenceDomain instance
     * @return saved ReferenceDomain instance.
     */
    ReferenceDomain saveOrUpdateReferenceDomain(ReferenceDomain domain);

    /**
     * Remove a ReferenceCode
     * @param referenceCode ReferenceCode instance
     */
    void removeReferenceCode(ReferenceCode referenceCode);

    /**
     * Remove a ReferenceCode by id
     * @param referenceCodeId ReferenceCode instance id
     */
    void removeReferenceCode(Long referenceCodeId);

    /**
     * @param domain belonging domain
     * @param code code to search
     * @return referenceCode instance
     */
    ReferenceCode getReferenceCode(String domain, String code);

}
